@@ -87,8 +87,7 @@ class TokenAuthHttpDownloader(HttpDownloader):
         if not self.token and not self.ansible_auth_url:
             return await super()._run(extra_data=extra_data)
         elif self.token and not self.ansible_auth_url:
-            # https://www.django-rest-framework.org/api-guide/authentication/#tokenauthentication
-            headers = {"Authorization": "Token {token}".format(token=self.token)}
+            headers = {"Authorization": "Bearer {token}".format(token=self.token)}
             return await self._run_with_additional_headers(headers)
         else:
             return await self._run_with_token_refresh_and_401_retry()
@@ -129,10 +128,7 @@ class TokenAuthHttpDownloader(HttpDownloader):
             DownloadResult: Contains information about the result. See the DownloadResult docs for
                  more information.
         """
-        await asyncio.sleep(0.75)
-        async with self.session.get(
-            self.url, headers=headers, proxy=self.proxy, proxy_auth=self.proxy_auth, auth=self.auth
-        ) as response:
+        async with self.session.get(self.url, headers=headers, proxy=self.proxy) as response:
             self.raise_for_status(response)
             to_return = await self._handle_response(response)
             await response.release()
@@ -160,14 +156,7 @@ class TokenAuthHttpDownloader(HttpDownloader):
                 "refresh_token": self.token,
             }
             url = self.ansible_auth_url
-            async with self.session.post(
-                url,
-                data=form_payload,
-                proxy=self.proxy,
-                proxy_auth=self.proxy_auth,
-                auth=self.auth,
-                raise_for_status=True,
-            ) as response:
+            async with self.session.post(url, data=form_payload, raise_for_status=True) as response:
                 token_data = await response.text()
 
             AUTH_TOKEN = json.loads(token_data)["access_token"]
@@ -215,10 +204,6 @@ class AnsibleDownloaderFactory(DownloaderFactory):
         options = {"session": self._session}
         if self._remote.proxy_url:
             options["proxy"] = self._remote.proxy_url
-            if self._remote.proxy_username and self._remote.proxy_password:
-                options["proxy_auth"] = BasicAuth(
-                    login=self._remote.proxy_username, password=self._remote.proxy_password
-                )
 
         if not self._remote.token and self._remote.username and self._remote.password:
             options["auth"] = BasicAuth(login=self._remote.username, password=self._remote.password)
